#define _CFFI_

/* We try to define Py_LIMITED_API before including Python.h.

   Mess: we can only define it if Py_DEBUG, Py_TRACE_REFS and
   Py_REF_DEBUG are not defined.  This is a best-effort approximation:
   we can learn about Py_DEBUG from pyconfig.h, but it is unclear if
   the same works for the other two macros.  Py_DEBUG implies them,
   but not the other way around.

   The implementation is messy (issue #350): on Windows, with _MSC_VER,
   we have to define Py_LIMITED_API even before including pyconfig.h.
   In that case, we guess what pyconfig.h will do to the macros above,
   and check our guess after the #include.

   Note that on Windows, with CPython 3.x, you need >= 3.5 and virtualenv
   version >= 16.0.0.  With older versions of either, you don't get a
   copy of PYTHON3.DLL in the virtualenv.  We can't check the version of
   CPython *before* we even include pyconfig.h.  ffi.set_source() puts
   a ``#define _CFFI_NO_LIMITED_API'' at the start of this file if it is
   running on Windows < 3.5, as an attempt at fixing it, but that's
   arguably wrong because it may not be the target version of Python.
   Still better than nothing I guess.  As another workaround, you can
   remove the definition of Py_LIMITED_API here.

   See also 'py_limited_api' in cffi/setuptools_ext.py.
*/
#if !defined(_CFFI_USE_EMBEDDING) && !defined(Py_LIMITED_API)
#  ifdef _MSC_VER
#    if !defined(_DEBUG) && !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#    include <pyconfig.h>
     /* sanity-check: Py_LIMITED_API will cause crashes if any of these
        are also defined.  Normally, the Python file PC/pyconfig.h does not
        cause any of these to be defined, with the exception that _DEBUG
        causes Py_DEBUG.  Double-check that. */
#    ifdef Py_LIMITED_API
#      if defined(Py_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_DEBUG, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_TRACE_REFS)
#        error "pyconfig.h unexpectedly defines Py_TRACE_REFS, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_REF_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_REF_DEBUG, but Py_LIMITED_API is set"
#      endif
#    endif
#  else
#    include <pyconfig.h>
#    if !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#  endif
#endif

#include <Python.h>
#ifdef __cplusplus
extern "C" {
#endif
#include <stddef.h>

/* This part is from file 'cffi/parse_c_type.h'.  It is copied at the
   beginning of C sources generated by CFFI's ffi.set_source(). */

typedef void *_cffi_opcode_t;

#define _CFFI_OP(opcode, arg)   (_cffi_opcode_t)(opcode | (((uintptr_t)(arg)) << 8))
#define _CFFI_GETOP(cffi_opcode)    ((unsigned char)(uintptr_t)cffi_opcode)
#define _CFFI_GETARG(cffi_opcode)   (((intptr_t)cffi_opcode) >> 8)

#define _CFFI_OP_PRIMITIVE       1
#define _CFFI_OP_POINTER         3
#define _CFFI_OP_ARRAY           5
#define _CFFI_OP_OPEN_ARRAY      7
#define _CFFI_OP_STRUCT_UNION    9
#define _CFFI_OP_ENUM           11
#define _CFFI_OP_FUNCTION       13
#define _CFFI_OP_FUNCTION_END   15
#define _CFFI_OP_NOOP           17
#define _CFFI_OP_BITFIELD       19
#define _CFFI_OP_TYPENAME       21
#define _CFFI_OP_CPYTHON_BLTN_V 23   // varargs
#define _CFFI_OP_CPYTHON_BLTN_N 25   // noargs
#define _CFFI_OP_CPYTHON_BLTN_O 27   // O  (i.e. a single arg)
#define _CFFI_OP_CONSTANT       29
#define _CFFI_OP_CONSTANT_INT   31
#define _CFFI_OP_GLOBAL_VAR     33
#define _CFFI_OP_DLOPEN_FUNC    35
#define _CFFI_OP_DLOPEN_CONST   37
#define _CFFI_OP_GLOBAL_VAR_F   39
#define _CFFI_OP_EXTERN_PYTHON  41

#define _CFFI_PRIM_VOID          0
#define _CFFI_PRIM_BOOL          1
#define _CFFI_PRIM_CHAR          2
#define _CFFI_PRIM_SCHAR         3
#define _CFFI_PRIM_UCHAR         4
#define _CFFI_PRIM_SHORT         5
#define _CFFI_PRIM_USHORT        6
#define _CFFI_PRIM_INT           7
#define _CFFI_PRIM_UINT          8
#define _CFFI_PRIM_LONG          9
#define _CFFI_PRIM_ULONG        10
#define _CFFI_PRIM_LONGLONG     11
#define _CFFI_PRIM_ULONGLONG    12
#define _CFFI_PRIM_FLOAT        13
#define _CFFI_PRIM_DOUBLE       14
#define _CFFI_PRIM_LONGDOUBLE   15

#define _CFFI_PRIM_WCHAR        16
#define _CFFI_PRIM_INT8         17
#define _CFFI_PRIM_UINT8        18
#define _CFFI_PRIM_INT16        19
#define _CFFI_PRIM_UINT16       20
#define _CFFI_PRIM_INT32        21
#define _CFFI_PRIM_UINT32       22
#define _CFFI_PRIM_INT64        23
#define _CFFI_PRIM_UINT64       24
#define _CFFI_PRIM_INTPTR       25
#define _CFFI_PRIM_UINTPTR      26
#define _CFFI_PRIM_PTRDIFF      27
#define _CFFI_PRIM_SIZE         28
#define _CFFI_PRIM_SSIZE        29
#define _CFFI_PRIM_INT_LEAST8   30
#define _CFFI_PRIM_UINT_LEAST8  31
#define _CFFI_PRIM_INT_LEAST16  32
#define _CFFI_PRIM_UINT_LEAST16 33
#define _CFFI_PRIM_INT_LEAST32  34
#define _CFFI_PRIM_UINT_LEAST32 35
#define _CFFI_PRIM_INT_LEAST64  36
#define _CFFI_PRIM_UINT_LEAST64 37
#define _CFFI_PRIM_INT_FAST8    38
#define _CFFI_PRIM_UINT_FAST8   39
#define _CFFI_PRIM_INT_FAST16   40
#define _CFFI_PRIM_UINT_FAST16  41
#define _CFFI_PRIM_INT_FAST32   42
#define _CFFI_PRIM_UINT_FAST32  43
#define _CFFI_PRIM_INT_FAST64   44
#define _CFFI_PRIM_UINT_FAST64  45
#define _CFFI_PRIM_INTMAX       46
#define _CFFI_PRIM_UINTMAX      47
#define _CFFI_PRIM_FLOATCOMPLEX 48
#define _CFFI_PRIM_DOUBLECOMPLEX 49
#define _CFFI_PRIM_CHAR16       50
#define _CFFI_PRIM_CHAR32       51

#define _CFFI__NUM_PRIM         52
#define _CFFI__UNKNOWN_PRIM           (-1)
#define _CFFI__UNKNOWN_FLOAT_PRIM     (-2)
#define _CFFI__UNKNOWN_LONG_DOUBLE    (-3)

#define _CFFI__IO_FILE_STRUCT         (-1)


struct _cffi_global_s {
    const char *name;
    void *address;
    _cffi_opcode_t type_op;
    void *size_or_direct_fn;  // OP_GLOBAL_VAR: size, or 0 if unknown
                              // OP_CPYTHON_BLTN_*: addr of direct function
};

struct _cffi_getconst_s {
    unsigned long long value;
    const struct _cffi_type_context_s *ctx;
    int gindex;
};

struct _cffi_struct_union_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_STRUCT_UNION
    int flags;               // _CFFI_F_* flags below
    size_t size;
    int alignment;
    int first_field_index;   // -> _cffi_fields array
    int num_fields;
};
#define _CFFI_F_UNION         0x01   // is a union, not a struct
#define _CFFI_F_CHECK_FIELDS  0x02   // complain if fields are not in the
                                     // "standard layout" or if some are missing
#define _CFFI_F_PACKED        0x04   // for CHECK_FIELDS, assume a packed struct
#define _CFFI_F_EXTERNAL      0x08   // in some other ffi.include()
#define _CFFI_F_OPAQUE        0x10   // opaque

struct _cffi_field_s {
    const char *name;
    size_t field_offset;
    size_t field_size;
    _cffi_opcode_t field_type_op;
};

struct _cffi_enum_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_ENUM
    int type_prim;           // _CFFI_PRIM_xxx
    const char *enumerators; // comma-delimited string
};

struct _cffi_typename_s {
    const char *name;
    int type_index;   /* if opaque, points to a possibly artificial
                         OP_STRUCT which is itself opaque */
};

struct _cffi_type_context_s {
    _cffi_opcode_t *types;
    const struct _cffi_global_s *globals;
    const struct _cffi_field_s *fields;
    const struct _cffi_struct_union_s *struct_unions;
    const struct _cffi_enum_s *enums;
    const struct _cffi_typename_s *typenames;
    int num_globals;
    int num_struct_unions;
    int num_enums;
    int num_typenames;
    const char *const *includes;
    int num_types;
    int flags;      /* future extension */
};

struct _cffi_parse_info_s {
    const struct _cffi_type_context_s *ctx;
    _cffi_opcode_t *output;
    unsigned int output_size;
    size_t error_location;
    const char *error_message;
};

struct _cffi_externpy_s {
    const char *name;
    size_t size_of_result;
    void *reserved1, *reserved2;
};

#ifdef _CFFI_INTERNAL
static int parse_c_type(struct _cffi_parse_info_s *info, const char *input);
static int search_in_globals(const struct _cffi_type_context_s *ctx,
                             const char *search, size_t search_len);
static int search_in_struct_unions(const struct _cffi_type_context_s *ctx,
                                   const char *search, size_t search_len);
#endif

/* this block of #ifs should be kept exactly identical between
   c/_cffi_backend.c, cffi/vengine_cpy.py, cffi/vengine_gen.py
   and cffi/_cffi_include.h */
#if defined(_MSC_VER)
# include <malloc.h>   /* for alloca() */
# if _MSC_VER < 1600   /* MSVC < 2010 */
   typedef __int8 int8_t;
   typedef __int16 int16_t;
   typedef __int32 int32_t;
   typedef __int64 int64_t;
   typedef unsigned __int8 uint8_t;
   typedef unsigned __int16 uint16_t;
   typedef unsigned __int32 uint32_t;
   typedef unsigned __int64 uint64_t;
   typedef __int8 int_least8_t;
   typedef __int16 int_least16_t;
   typedef __int32 int_least32_t;
   typedef __int64 int_least64_t;
   typedef unsigned __int8 uint_least8_t;
   typedef unsigned __int16 uint_least16_t;
   typedef unsigned __int32 uint_least32_t;
   typedef unsigned __int64 uint_least64_t;
   typedef __int8 int_fast8_t;
   typedef __int16 int_fast16_t;
   typedef __int32 int_fast32_t;
   typedef __int64 int_fast64_t;
   typedef unsigned __int8 uint_fast8_t;
   typedef unsigned __int16 uint_fast16_t;
   typedef unsigned __int32 uint_fast32_t;
   typedef unsigned __int64 uint_fast64_t;
   typedef __int64 intmax_t;
   typedef unsigned __int64 uintmax_t;
# else
#  include <stdint.h>
# endif
# if _MSC_VER < 1800   /* MSVC < 2013 */
#  ifndef __cplusplus
    typedef unsigned char _Bool;
#  endif
# endif
#else
# include <stdint.h>
# if (defined (__SVR4) && defined (__sun)) || defined(_AIX) || defined(__hpux)
#  include <alloca.h>
# endif
#endif

#ifdef __GNUC__
# define _CFFI_UNUSED_FN  __attribute__((unused))
#else
# define _CFFI_UNUSED_FN  /* nothing */
#endif

#ifdef __cplusplus
# ifndef _Bool
   typedef bool _Bool;   /* semi-hackish: C++ has no _Bool; bool is builtin */
# endif
#endif

/**********  CPython-specific section  **********/
#ifndef PYPY_VERSION


#if PY_MAJOR_VERSION >= 3
# define PyInt_FromLong PyLong_FromLong
#endif

#define _cffi_from_c_double PyFloat_FromDouble
#define _cffi_from_c_float PyFloat_FromDouble
#define _cffi_from_c_long PyInt_FromLong
#define _cffi_from_c_ulong PyLong_FromUnsignedLong
#define _cffi_from_c_longlong PyLong_FromLongLong
#define _cffi_from_c_ulonglong PyLong_FromUnsignedLongLong
#define _cffi_from_c__Bool PyBool_FromLong

#define _cffi_to_c_double PyFloat_AsDouble
#define _cffi_to_c_float PyFloat_AsDouble

#define _cffi_from_c_int(x, type)                                        \
    (((type)-1) > 0 ? /* unsigned */                                     \
        (sizeof(type) < sizeof(long) ?                                   \
            PyInt_FromLong((long)x) :                                    \
         sizeof(type) == sizeof(long) ?                                  \
            PyLong_FromUnsignedLong((unsigned long)x) :                  \
            PyLong_FromUnsignedLongLong((unsigned long long)x)) :        \
        (sizeof(type) <= sizeof(long) ?                                  \
            PyInt_FromLong((long)x) :                                    \
            PyLong_FromLongLong((long long)x)))

#define _cffi_to_c_int(o, type)                                          \
    ((type)(                                                             \
     sizeof(type) == 1 ? (((type)-1) > 0 ? (type)_cffi_to_c_u8(o)        \
                                         : (type)_cffi_to_c_i8(o)) :     \
     sizeof(type) == 2 ? (((type)-1) > 0 ? (type)_cffi_to_c_u16(o)       \
                                         : (type)_cffi_to_c_i16(o)) :    \
     sizeof(type) == 4 ? (((type)-1) > 0 ? (type)_cffi_to_c_u32(o)       \
                                         : (type)_cffi_to_c_i32(o)) :    \
     sizeof(type) == 8 ? (((type)-1) > 0 ? (type)_cffi_to_c_u64(o)       \
                                         : (type)_cffi_to_c_i64(o)) :    \
     (Py_FatalError("unsupported size for type " #type), (type)0)))

#define _cffi_to_c_i8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[1])
#define _cffi_to_c_u8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[2])
#define _cffi_to_c_i16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[3])
#define _cffi_to_c_u16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[4])
#define _cffi_to_c_i32                                                   \
                 ((int(*)(PyObject *))_cffi_exports[5])
#define _cffi_to_c_u32                                                   \
                 ((unsigned int(*)(PyObject *))_cffi_exports[6])
#define _cffi_to_c_i64                                                   \
                 ((long long(*)(PyObject *))_cffi_exports[7])
#define _cffi_to_c_u64                                                   \
                 ((unsigned long long(*)(PyObject *))_cffi_exports[8])
#define _cffi_to_c_char                                                  \
                 ((int(*)(PyObject *))_cffi_exports[9])
#define _cffi_from_c_pointer                                             \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[10])
#define _cffi_to_c_pointer                                               \
    ((char *(*)(PyObject *, struct _cffi_ctypedescr *))_cffi_exports[11])
#define _cffi_get_struct_layout                                          \
    not used any more
#define _cffi_restore_errno                                              \
    ((void(*)(void))_cffi_exports[13])
#define _cffi_save_errno                                                 \
    ((void(*)(void))_cffi_exports[14])
#define _cffi_from_c_char                                                \
    ((PyObject *(*)(char))_cffi_exports[15])
#define _cffi_from_c_deref                                               \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[16])
#define _cffi_to_c                                                       \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[17])
#define _cffi_from_c_struct                                              \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[18])
#define _cffi_to_c_wchar_t                                               \
    ((_cffi_wchar_t(*)(PyObject *))_cffi_exports[19])
#define _cffi_from_c_wchar_t                                             \
    ((PyObject *(*)(_cffi_wchar_t))_cffi_exports[20])
#define _cffi_to_c_long_double                                           \
    ((long double(*)(PyObject *))_cffi_exports[21])
#define _cffi_to_c__Bool                                                 \
    ((_Bool(*)(PyObject *))_cffi_exports[22])
#define _cffi_prepare_pointer_call_argument                              \
    ((Py_ssize_t(*)(struct _cffi_ctypedescr *,                           \
                    PyObject *, char **))_cffi_exports[23])
#define _cffi_convert_array_from_object                                  \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[24])
#define _CFFI_CPIDX  25
#define _cffi_call_python                                                \
    ((void(*)(struct _cffi_externpy_s *, char *))_cffi_exports[_CFFI_CPIDX])
#define _cffi_to_c_wchar3216_t                                           \
    ((int(*)(PyObject *))_cffi_exports[26])
#define _cffi_from_c_wchar3216_t                                         \
    ((PyObject *(*)(int))_cffi_exports[27])
#define _CFFI_NUM_EXPORTS 28

struct _cffi_ctypedescr;

static void *_cffi_exports[_CFFI_NUM_EXPORTS];

#define _cffi_type(index)   (                           \
    assert((((uintptr_t)_cffi_types[index]) & 1) == 0), \
    (struct _cffi_ctypedescr *)_cffi_types[index])

static PyObject *_cffi_init(const char *module_name, Py_ssize_t version,
                            const struct _cffi_type_context_s *ctx)
{
    PyObject *module, *o_arg, *new_module;
    void *raw[] = {
        (void *)module_name,
        (void *)version,
        (void *)_cffi_exports,
        (void *)ctx,
    };

    module = PyImport_ImportModule("_cffi_backend");
    if (module == NULL)
        goto failure;

    o_arg = PyLong_FromVoidPtr((void *)raw);
    if (o_arg == NULL)
        goto failure;

    new_module = PyObject_CallMethod(
        module, (char *)"_init_cffi_1_0_external_module", (char *)"O", o_arg);

    Py_DECREF(o_arg);
    Py_DECREF(module);
    return new_module;

  failure:
    Py_XDECREF(module);
    return NULL;
}


#ifdef HAVE_WCHAR_H
typedef wchar_t _cffi_wchar_t;
#else
typedef uint16_t _cffi_wchar_t;   /* same random pick as _cffi_backend.c */
#endif

_CFFI_UNUSED_FN static uint16_t _cffi_to_c_char16_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return (uint16_t)_cffi_to_c_wchar_t(o);
    else
        return (uint16_t)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char16_t(uint16_t x)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

_CFFI_UNUSED_FN static int _cffi_to_c_char32_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return (int)_cffi_to_c_wchar_t(o);
    else
        return (int)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char32_t(unsigned int x)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

union _cffi_union_alignment_u {
    unsigned char m_char;
    unsigned short m_short;
    unsigned int m_int;
    unsigned long m_long;
    unsigned long long m_longlong;
    float m_float;
    double m_double;
    long double m_longdouble;
};

struct _cffi_freeme_s {
    struct _cffi_freeme_s *next;
    union _cffi_union_alignment_u alignment;
};

_CFFI_UNUSED_FN static int
_cffi_convert_array_argument(struct _cffi_ctypedescr *ctptr, PyObject *arg,
                             char **output_data, Py_ssize_t datasize,
                             struct _cffi_freeme_s **freeme)
{
    char *p;
    if (datasize < 0)
        return -1;

    p = *output_data;
    if (p == NULL) {
        struct _cffi_freeme_s *fp = (struct _cffi_freeme_s *)PyObject_Malloc(
            offsetof(struct _cffi_freeme_s, alignment) + (size_t)datasize);
        if (fp == NULL)
            return -1;
        fp->next = *freeme;
        *freeme = fp;
        p = *output_data = (char *)&fp->alignment;
    }
    memset((void *)p, 0, (size_t)datasize);
    return _cffi_convert_array_from_object(p, ctptr, arg);
}

_CFFI_UNUSED_FN static void
_cffi_free_array_arguments(struct _cffi_freeme_s *freeme)
{
    do {
        void *p = (void *)freeme;
        freeme = freeme->next;
        PyObject_Free(p);
    } while (freeme != NULL);
}

/**********  end CPython-specific section  **********/
#else
_CFFI_UNUSED_FN
static void (*_cffi_call_python_org)(struct _cffi_externpy_s *, char *);
# define _cffi_call_python  _cffi_call_python_org
#endif


#define _cffi_array_len(array)   (sizeof(array) / sizeof((array)[0]))

#define _cffi_prim_int(size, sign)                                      \
    ((size) == 1 ? ((sign) ? _CFFI_PRIM_INT8  : _CFFI_PRIM_UINT8)  :    \
     (size) == 2 ? ((sign) ? _CFFI_PRIM_INT16 : _CFFI_PRIM_UINT16) :    \
     (size) == 4 ? ((sign) ? _CFFI_PRIM_INT32 : _CFFI_PRIM_UINT32) :    \
     (size) == 8 ? ((sign) ? _CFFI_PRIM_INT64 : _CFFI_PRIM_UINT64) :    \
     _CFFI__UNKNOWN_PRIM)

#define _cffi_prim_float(size)                                          \
    ((size) == sizeof(float) ? _CFFI_PRIM_FLOAT :                       \
     (size) == sizeof(double) ? _CFFI_PRIM_DOUBLE :                     \
     (size) == sizeof(long double) ? _CFFI__UNKNOWN_LONG_DOUBLE :       \
     _CFFI__UNKNOWN_FLOAT_PRIM)

#define _cffi_check_int(got, got_nonpos, expected)      \
    ((got_nonpos) == (expected <= 0) &&                 \
     (got) == (unsigned long long)expected)

#ifdef MS_WIN32
# define _cffi_stdcall  __stdcall
#else
# define _cffi_stdcall  /* nothing */
#endif

#ifdef __cplusplus
}
#endif

/************************************************************/

#include "/etc/traceable/libtraceable.h"

/************************************************************/

static void *_cffi_types[] = {
/*  0 */ _CFFI_OP(_CFFI_OP_FUNCTION, 50), // TRACEABLE_RET()(char const *, char const * *, int, int *)
/*  1 */ _CFFI_OP(_CFFI_OP_POINTER, 53), // char const *
/*  2 */ _CFFI_OP(_CFFI_OP_POINTER, 1), // char const * *
/*  3 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7), // int
/*  4 */ _CFFI_OP(_CFFI_OP_POINTER, 3), // int *
/*  5 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/*  6 */ _CFFI_OP(_CFFI_OP_FUNCTION, 50), // TRACEABLE_RET()(char const *, int, int, char * *)
/*  7 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/*  8 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/*  9 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 10 */ _CFFI_OP(_CFFI_OP_POINTER, 52), // char * *
/* 11 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 12 */ _CFFI_OP(_CFFI_OP_FUNCTION, 50), // TRACEABLE_RET()(char const *, void * *)
/* 13 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 14 */ _CFFI_OP(_CFFI_OP_POINTER, 30), // void * *
/* 15 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 16 */ _CFFI_OP(_CFFI_OP_FUNCTION, 50), // TRACEABLE_RET()(modsecurity_rule_matches)
/* 17 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 1), // modsecurity_rule_matches
/* 18 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 19 */ _CFFI_OP(_CFFI_OP_FUNCTION, 50), // TRACEABLE_RET()(traceable_attributes)
/* 20 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 5), // traceable_attributes
/* 21 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 22 */ _CFFI_OP(_CFFI_OP_FUNCTION, 50), // TRACEABLE_RET()(traceable_libtraceable_config, void * *)
/* 23 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 7), // traceable_libtraceable_config
/* 24 */ _CFFI_OP(_CFFI_OP_NOOP, 14),
/* 25 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 26 */ _CFFI_OP(_CFFI_OP_FUNCTION, 50), // TRACEABLE_RET()(traceable_process_request_result)
/* 27 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 11), // traceable_process_request_result
/* 28 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 29 */ _CFFI_OP(_CFFI_OP_FUNCTION, 50), // TRACEABLE_RET()(void *)
/* 30 */ _CFFI_OP(_CFFI_OP_POINTER, 74), // void *
/* 31 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 32 */ _CFFI_OP(_CFFI_OP_FUNCTION, 50), // TRACEABLE_RET()(void *, traceable_attributes, modsecurity_rule_matches *)
/* 33 */ _CFFI_OP(_CFFI_OP_NOOP, 30),
/* 34 */ _CFFI_OP(_CFFI_OP_NOOP, 20),
/* 35 */ _CFFI_OP(_CFFI_OP_POINTER, 17), // modsecurity_rule_matches *
/* 36 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 37 */ _CFFI_OP(_CFFI_OP_FUNCTION, 50), // TRACEABLE_RET()(void *, traceable_attributes, traceable_attributes *)
/* 38 */ _CFFI_OP(_CFFI_OP_NOOP, 30),
/* 39 */ _CFFI_OP(_CFFI_OP_NOOP, 20),
/* 40 */ _CFFI_OP(_CFFI_OP_POINTER, 20), // traceable_attributes *
/* 41 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 42 */ _CFFI_OP(_CFFI_OP_FUNCTION, 50), // TRACEABLE_RET()(void *, traceable_attributes, traceable_process_request_result *)
/* 43 */ _CFFI_OP(_CFFI_OP_NOOP, 30),
/* 44 */ _CFFI_OP(_CFFI_OP_NOOP, 20),
/* 45 */ _CFFI_OP(_CFFI_OP_POINTER, 27), // traceable_process_request_result *
/* 46 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 47 */ _CFFI_OP(_CFFI_OP_FUNCTION, 23), // traceable_libtraceable_config()(void)
/* 48 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 49 */ _CFFI_OP(_CFFI_OP_ENUM, 0), // TRACEABLE_LOG_MODE
/* 50 */ _CFFI_OP(_CFFI_OP_ENUM, 1), // TRACEABLE_RET
/* 51 */ _CFFI_OP(_CFFI_OP_ENUM, 2), // TRACEABLE_SPAN_TYPE
/* 52 */ _CFFI_OP(_CFFI_OP_POINTER, 53), // char *
/* 53 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 2), // char
/* 54 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 9), // long
/* 55 */ _CFFI_OP(_CFFI_OP_POINTER, 56), // modsecurity_rule_match *
/* 56 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 0), // modsecurity_rule_match
/* 57 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 5), // short
/* 58 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 3), // signed char
/* 59 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 2), // traceable_agent_config
/* 60 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 3), // traceable_api_discovery_config
/* 61 */ _CFFI_OP(_CFFI_OP_POINTER, 62), // traceable_attribute const *
/* 62 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 4), // traceable_attribute
/* 63 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 6), // traceable_blocking_config
/* 64 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 8), // traceable_log_configuration
/* 65 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 9), // traceable_modsecurity_config
/* 66 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 10), // traceable_opa_config
/* 67 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 12), // traceable_rangeblocking_config
/* 68 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 13), // traceable_remote_config
/* 69 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 14), // traceable_sampling_config
/* 70 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 4), // unsigned char
/* 71 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 8), // unsigned int
/* 72 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 10), // unsigned long
/* 73 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 6), // unsigned short
/* 74 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 0), // void
};

static int _cffi_const_TRACEABLE_LOG_NONE(unsigned long long *o)
{
  int n = (TRACEABLE_LOG_NONE) <= 0;
  *o = (unsigned long long)((TRACEABLE_LOG_NONE) | 0);  /* check that TRACEABLE_LOG_NONE is an integer */
  return n;
}

static int _cffi_const_TRACEABLE_LOG_STDOUT(unsigned long long *o)
{
  int n = (TRACEABLE_LOG_STDOUT) <= 0;
  *o = (unsigned long long)((TRACEABLE_LOG_STDOUT) | 0);  /* check that TRACEABLE_LOG_STDOUT is an integer */
  return n;
}

static int _cffi_const_TRACEABLE_SUCCESS(unsigned long long *o)
{
  int n = (TRACEABLE_SUCCESS) <= 0;
  *o = (unsigned long long)((TRACEABLE_SUCCESS) | 0);  /* check that TRACEABLE_SUCCESS is an integer */
  return n;
}

static int _cffi_const_TRACEABLE_FAIL(unsigned long long *o)
{
  int n = (TRACEABLE_FAIL) <= 0;
  *o = (unsigned long long)((TRACEABLE_FAIL) | 0);  /* check that TRACEABLE_FAIL is an integer */
  return n;
}

static int _cffi_const_TRACEABLE_NO_SPAN(unsigned long long *o)
{
  int n = (TRACEABLE_NO_SPAN) <= 0;
  *o = (unsigned long long)((TRACEABLE_NO_SPAN) | 0);  /* check that TRACEABLE_NO_SPAN is an integer */
  return n;
}

static int _cffi_const_TRACEABLE_BARE_SPAN(unsigned long long *o)
{
  int n = (TRACEABLE_BARE_SPAN) <= 0;
  *o = (unsigned long long)((TRACEABLE_BARE_SPAN) | 0);  /* check that TRACEABLE_BARE_SPAN is an integer */
  return n;
}

static int _cffi_const_TRACEABLE_FULL_SPAN(unsigned long long *o)
{
  int n = (TRACEABLE_FULL_SPAN) <= 0;
  *o = (unsigned long long)((TRACEABLE_FULL_SPAN) | 0);  /* check that TRACEABLE_FULL_SPAN is an integer */
  return n;
}

_CFFI_UNUSED_FN
static void _cffi_checkfld_typedef_modsecurity_rule_match(modsecurity_rule_match *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { char * *tmp = &p->rule_id; (void)tmp; }
  { char * *tmp = &p->rule_message; (void)tmp; }
  { char * *tmp = &p->match_message; (void)tmp; }
  { char * *tmp = &p->match_attribute; (void)tmp; }
  (void)((p->paranoia_level) | 0);  /* check that 'modsecurity_rule_match.paranoia_level' is an integer */
  { char * *tmp = &p->rule_uuid; (void)tmp; }
}
struct _cffi_align_typedef_modsecurity_rule_match { char x; modsecurity_rule_match y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_typedef_modsecurity_rule_matches(modsecurity_rule_matches *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->count) | 0);  /* check that 'modsecurity_rule_matches.count' is an integer */
  { modsecurity_rule_match * *tmp = &p->match_arr; (void)tmp; }
}
struct _cffi_align_typedef_modsecurity_rule_matches { char x; modsecurity_rule_matches y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_typedef_traceable_agent_config(traceable_agent_config *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { char const * *tmp = &p->service_name; (void)tmp; }
}
struct _cffi_align_typedef_traceable_agent_config { char x; traceable_agent_config y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_typedef_traceable_api_discovery_config(traceable_api_discovery_config *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->enabled) | 0);  /* check that 'traceable_api_discovery_config.enabled' is an integer */
}
struct _cffi_align_typedef_traceable_api_discovery_config { char x; traceable_api_discovery_config y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_typedef_traceable_attribute(traceable_attribute *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { char const * *tmp = &p->key; (void)tmp; }
  { char const * *tmp = &p->value; (void)tmp; }
}
struct _cffi_align_typedef_traceable_attribute { char x; traceable_attribute y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_typedef_traceable_attributes(traceable_attributes *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->count) | 0);  /* check that 'traceable_attributes.count' is an integer */
  { traceable_attribute const * *tmp = &p->attribute_array; (void)tmp; }
}
struct _cffi_align_typedef_traceable_attributes { char x; traceable_attributes y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_typedef_traceable_blocking_config(traceable_blocking_config *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->enabled) | 0);  /* check that 'traceable_blocking_config.enabled' is an integer */
  { traceable_opa_config *tmp = &p->opa_config; (void)tmp; }
  { traceable_modsecurity_config *tmp = &p->modsecurity_config; (void)tmp; }
  { traceable_rangeblocking_config *tmp = &p->rb_config; (void)tmp; }
  (void)((p->evaluate_body) | 0);  /* check that 'traceable_blocking_config.evaluate_body' is an integer */
  (void)((p->skip_internal_request) | 0);  /* check that 'traceable_blocking_config.skip_internal_request' is an integer */
  (void)((p->max_recursion_depth) | 0);  /* check that 'traceable_blocking_config.max_recursion_depth' is an integer */
}
struct _cffi_align_typedef_traceable_blocking_config { char x; traceable_blocking_config y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_typedef_traceable_libtraceable_config(traceable_libtraceable_config *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { traceable_log_configuration *tmp = &p->log_config; (void)tmp; }
  { traceable_remote_config *tmp = &p->remote_config; (void)tmp; }
  { traceable_blocking_config *tmp = &p->blocking_config; (void)tmp; }
  { traceable_agent_config *tmp = &p->agent_config; (void)tmp; }
  { traceable_api_discovery_config *tmp = &p->api_discovery_config; (void)tmp; }
  { traceable_sampling_config *tmp = &p->sampling_config; (void)tmp; }
}
struct _cffi_align_typedef_traceable_libtraceable_config { char x; traceable_libtraceable_config y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_typedef_traceable_log_configuration(traceable_log_configuration *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { TRACEABLE_LOG_MODE *tmp = &p->mode; (void)tmp; }
}
struct _cffi_align_typedef_traceable_log_configuration { char x; traceable_log_configuration y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_typedef_traceable_modsecurity_config(traceable_modsecurity_config *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->enabled) | 0);  /* check that 'traceable_modsecurity_config.enabled' is an integer */
}
struct _cffi_align_typedef_traceable_modsecurity_config { char x; traceable_modsecurity_config y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_typedef_traceable_opa_config(traceable_opa_config *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->enabled) | 0);  /* check that 'traceable_opa_config.enabled' is an integer */
  { char const * *tmp = &p->opa_server_url; (void)tmp; }
  { char const * *tmp = &p->logging_dir; (void)tmp; }
  { char const * *tmp = &p->logging_file_prefix; (void)tmp; }
  { char const * *tmp = &p->cert_file; (void)tmp; }
  (void)((p->use_secure_connection) | 0);  /* check that 'traceable_opa_config.use_secure_connection' is an integer */
  (void)((p->log_to_console) | 0);  /* check that 'traceable_opa_config.log_to_console' is an integer */
  (void)((p->skip_verify) | 0);  /* check that 'traceable_opa_config.skip_verify' is an integer */
  (void)((p->min_delay) | 0);  /* check that 'traceable_opa_config.min_delay' is an integer */
  (void)((p->max_delay) | 0);  /* check that 'traceable_opa_config.max_delay' is an integer */
  (void)((p->debug_log) | 0);  /* check that 'traceable_opa_config.debug_log' is an integer */
}
struct _cffi_align_typedef_traceable_opa_config { char x; traceable_opa_config y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_typedef_traceable_process_request_result(traceable_process_request_result *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->block) | 0);  /* check that 'traceable_process_request_result.block' is an integer */
  { TRACEABLE_SPAN_TYPE *tmp = &p->span_type; (void)tmp; }
  (void)((p->propagate) | 0);  /* check that 'traceable_process_request_result.propagate' is an integer */
  { traceable_attributes *tmp = &p->attributes; (void)tmp; }
}
struct _cffi_align_typedef_traceable_process_request_result { char x; traceable_process_request_result y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_typedef_traceable_rangeblocking_config(traceable_rangeblocking_config *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->enabled) | 0);  /* check that 'traceable_rangeblocking_config.enabled' is an integer */
}
struct _cffi_align_typedef_traceable_rangeblocking_config { char x; traceable_rangeblocking_config y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_typedef_traceable_remote_config(traceable_remote_config *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->enabled) | 0);  /* check that 'traceable_remote_config.enabled' is an integer */
  { char const * *tmp = &p->remote_endpoint; (void)tmp; }
  (void)((p->poll_period_sec) | 0);  /* check that 'traceable_remote_config.poll_period_sec' is an integer */
  { char const * *tmp = &p->cert_file; (void)tmp; }
  (void)((p->use_secure_connection) | 0);  /* check that 'traceable_remote_config.use_secure_connection' is an integer */
  (void)((p->grpc_max_call_recv_msg_size) | 0);  /* check that 'traceable_remote_config.grpc_max_call_recv_msg_size' is an integer */
  { char const * *tmp = &p->cert_string; (void)tmp; }
}
struct _cffi_align_typedef_traceable_remote_config { char x; traceable_remote_config y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_typedef_traceable_sampling_config(traceable_sampling_config *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->enabled) | 0);  /* check that 'traceable_sampling_config.enabled' is an integer */
}
struct _cffi_align_typedef_traceable_sampling_config { char x; traceable_sampling_config y; };

static traceable_libtraceable_config _cffi_d_init_libtraceable_config(void)
{
  return init_libtraceable_config();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_init_libtraceable_config(PyObject *self, PyObject *noarg)
{
  traceable_libtraceable_config result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = init_libtraceable_config(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(23));
  return pyresult;
}
#else
static void _cffi_f_init_libtraceable_config(traceable_libtraceable_config *result)
{
  { *result = init_libtraceable_config(); }
}
#endif

static TRACEABLE_RET _cffi_d_modsecurity_cleanup_rule_engine(void * x0)
{
  return modsecurity_cleanup_rule_engine(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_modsecurity_cleanup_rule_engine(PyObject *self, PyObject *arg0)
{
  void * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  TRACEABLE_RET result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(30), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(30), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = modsecurity_cleanup_rule_engine(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(50));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_modsecurity_cleanup_rule_engine _cffi_d_modsecurity_cleanup_rule_engine
#endif

static TRACEABLE_RET _cffi_d_modsecurity_cleanup_rule_matches(modsecurity_rule_matches x0)
{
  return modsecurity_cleanup_rule_matches(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_modsecurity_cleanup_rule_matches(PyObject *self, PyObject *arg0)
{
  modsecurity_rule_matches x0;
  TRACEABLE_RET result;
  PyObject *pyresult;

  if (_cffi_to_c((char *)&x0, _cffi_type(17), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = modsecurity_cleanup_rule_matches(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(50));
  return pyresult;
}
#else
static TRACEABLE_RET _cffi_f_modsecurity_cleanup_rule_matches(modsecurity_rule_matches *x0)
{
  TRACEABLE_RET result;
  { result = modsecurity_cleanup_rule_matches(*x0); }
  return result;
}
#endif

static TRACEABLE_RET _cffi_d_modsecurity_new_rule_engine(char const * x0, void * * x1)
{
  return modsecurity_new_rule_engine(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_modsecurity_new_rule_engine(PyObject *self, PyObject *args)
{
  char const * x0;
  void * * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  TRACEABLE_RET result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "modsecurity_new_rule_engine", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(14), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (void * *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(14), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = modsecurity_new_rule_engine(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(50));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_modsecurity_new_rule_engine _cffi_d_modsecurity_new_rule_engine
#endif

static TRACEABLE_RET _cffi_d_modsecurity_process_attributes(void * x0, traceable_attributes x1, modsecurity_rule_matches * x2)
{
  return modsecurity_process_attributes(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_modsecurity_process_attributes(PyObject *self, PyObject *args)
{
  void * x0;
  traceable_attributes x1;
  modsecurity_rule_matches * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  TRACEABLE_RET result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "modsecurity_process_attributes", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(30), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(30), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  if (_cffi_to_c((char *)&x1, _cffi_type(20), arg1) < 0)
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(35), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (modsecurity_rule_matches *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(35), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = modsecurity_process_attributes(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(50));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
static TRACEABLE_RET _cffi_f_modsecurity_process_attributes(void * x0, traceable_attributes *x1, modsecurity_rule_matches * x2)
{
  TRACEABLE_RET result;
  { result = modsecurity_process_attributes(x0, *x1, x2); }
  return result;
}
#endif

static TRACEABLE_RET _cffi_d_traceable_barespan_attributes(void * x0, traceable_attributes x1, traceable_attributes * x2)
{
  return traceable_barespan_attributes(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_traceable_barespan_attributes(PyObject *self, PyObject *args)
{
  void * x0;
  traceable_attributes x1;
  traceable_attributes * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  TRACEABLE_RET result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "traceable_barespan_attributes", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(30), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(30), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  if (_cffi_to_c((char *)&x1, _cffi_type(20), arg1) < 0)
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(40), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (traceable_attributes *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(40), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = traceable_barespan_attributes(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(50));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
static TRACEABLE_RET _cffi_f_traceable_barespan_attributes(void * x0, traceable_attributes *x1, traceable_attributes * x2)
{
  TRACEABLE_RET result;
  { result = traceable_barespan_attributes(x0, *x1, x2); }
  return result;
}
#endif

static TRACEABLE_RET _cffi_d_traceable_decode_protobuf(char const * x0, int x1, int x2, char * * x3)
{
  return traceable_decode_protobuf(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_traceable_decode_protobuf(PyObject *self, PyObject *args)
{
  char const * x0;
  int x1;
  int x2;
  char * * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  TRACEABLE_RET result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "traceable_decode_protobuf", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(10), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (char * *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(10), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = traceable_decode_protobuf(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(50));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_traceable_decode_protobuf _cffi_d_traceable_decode_protobuf
#endif

static TRACEABLE_RET _cffi_d_traceable_delete_barespan_attributes_result(traceable_attributes x0)
{
  return traceable_delete_barespan_attributes_result(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_traceable_delete_barespan_attributes_result(PyObject *self, PyObject *arg0)
{
  traceable_attributes x0;
  TRACEABLE_RET result;
  PyObject *pyresult;

  if (_cffi_to_c((char *)&x0, _cffi_type(20), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = traceable_delete_barespan_attributes_result(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(50));
  return pyresult;
}
#else
static TRACEABLE_RET _cffi_f_traceable_delete_barespan_attributes_result(traceable_attributes *x0)
{
  TRACEABLE_RET result;
  { result = traceable_delete_barespan_attributes_result(*x0); }
  return result;
}
#endif

static TRACEABLE_RET _cffi_d_traceable_delete_libtraceable(void * x0)
{
  return traceable_delete_libtraceable(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_traceable_delete_libtraceable(PyObject *self, PyObject *arg0)
{
  void * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  TRACEABLE_RET result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(30), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(30), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = traceable_delete_libtraceable(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(50));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_traceable_delete_libtraceable _cffi_d_traceable_delete_libtraceable
#endif

static TRACEABLE_RET _cffi_d_traceable_delete_process_request_result_data(traceable_process_request_result x0)
{
  return traceable_delete_process_request_result_data(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_traceable_delete_process_request_result_data(PyObject *self, PyObject *arg0)
{
  traceable_process_request_result x0;
  TRACEABLE_RET result;
  PyObject *pyresult;

  if (_cffi_to_c((char *)&x0, _cffi_type(27), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = traceable_delete_process_request_result_data(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(50));
  return pyresult;
}
#else
static TRACEABLE_RET _cffi_f_traceable_delete_process_request_result_data(traceable_process_request_result *x0)
{
  TRACEABLE_RET result;
  { result = traceable_delete_process_request_result_data(*x0); }
  return result;
}
#endif

static TRACEABLE_RET _cffi_d_traceable_is_content_type_capturable(char const * x0, char const * * x1, int x2, int * x3)
{
  return traceable_is_content_type_capturable(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_traceable_is_content_type_capturable(PyObject *self, PyObject *args)
{
  char const * x0;
  char const * * x1;
  int x2;
  int * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  TRACEABLE_RET result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "traceable_is_content_type_capturable", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const * *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = traceable_is_content_type_capturable(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(50));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_traceable_is_content_type_capturable _cffi_d_traceable_is_content_type_capturable
#endif

static TRACEABLE_RET _cffi_d_traceable_new_libtraceable(traceable_libtraceable_config x0, void * * x1)
{
  return traceable_new_libtraceable(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_traceable_new_libtraceable(PyObject *self, PyObject *args)
{
  traceable_libtraceable_config x0;
  void * * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  TRACEABLE_RET result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "traceable_new_libtraceable", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(23), arg0) < 0)
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(14), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (void * *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(14), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = traceable_new_libtraceable(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(50));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
static TRACEABLE_RET _cffi_f_traceable_new_libtraceable(traceable_libtraceable_config *x0, void * * x1)
{
  TRACEABLE_RET result;
  { result = traceable_new_libtraceable(*x0, x1); }
  return result;
}
#endif

static TRACEABLE_RET _cffi_d_traceable_process_request(void * x0, traceable_attributes x1, traceable_process_request_result * x2)
{
  return traceable_process_request(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_traceable_process_request(PyObject *self, PyObject *args)
{
  void * x0;
  traceable_attributes x1;
  traceable_process_request_result * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  TRACEABLE_RET result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "traceable_process_request", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(30), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(30), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  if (_cffi_to_c((char *)&x1, _cffi_type(20), arg1) < 0)
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(45), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (traceable_process_request_result *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(45), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = traceable_process_request(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(50));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
static TRACEABLE_RET _cffi_f_traceable_process_request(void * x0, traceable_attributes *x1, traceable_process_request_result * x2)
{
  TRACEABLE_RET result;
  { result = traceable_process_request(x0, *x1, x2); }
  return result;
}
#endif

static TRACEABLE_RET _cffi_d_traceable_process_request_body(void * x0, traceable_attributes x1, traceable_process_request_result * x2)
{
  return traceable_process_request_body(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_traceable_process_request_body(PyObject *self, PyObject *args)
{
  void * x0;
  traceable_attributes x1;
  traceable_process_request_result * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  TRACEABLE_RET result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "traceable_process_request_body", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(30), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(30), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  if (_cffi_to_c((char *)&x1, _cffi_type(20), arg1) < 0)
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(45), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (traceable_process_request_result *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(45), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = traceable_process_request_body(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(50));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
static TRACEABLE_RET _cffi_f_traceable_process_request_body(void * x0, traceable_attributes *x1, traceable_process_request_result * x2)
{
  TRACEABLE_RET result;
  { result = traceable_process_request_body(x0, *x1, x2); }
  return result;
}
#endif

static TRACEABLE_RET _cffi_d_traceable_process_request_headers(void * x0, traceable_attributes x1, traceable_process_request_result * x2)
{
  return traceable_process_request_headers(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_traceable_process_request_headers(PyObject *self, PyObject *args)
{
  void * x0;
  traceable_attributes x1;
  traceable_process_request_result * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  TRACEABLE_RET result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "traceable_process_request_headers", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(30), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(30), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  if (_cffi_to_c((char *)&x1, _cffi_type(20), arg1) < 0)
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(45), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (traceable_process_request_result *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(45), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = traceable_process_request_headers(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(50));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
static TRACEABLE_RET _cffi_f_traceable_process_request_headers(void * x0, traceable_attributes *x1, traceable_process_request_result * x2)
{
  TRACEABLE_RET result;
  { result = traceable_process_request_headers(x0, *x1, x2); }
  return result;
}
#endif

static TRACEABLE_RET _cffi_d_traceable_start_libtraceable(void * x0)
{
  return traceable_start_libtraceable(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_traceable_start_libtraceable(PyObject *self, PyObject *arg0)
{
  void * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  TRACEABLE_RET result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(30), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(30), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = traceable_start_libtraceable(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(50));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_traceable_start_libtraceable _cffi_d_traceable_start_libtraceable
#endif

static const struct _cffi_global_s _cffi_globals[] = {
  { "TRACEABLE_BARE_SPAN", (void *)_cffi_const_TRACEABLE_BARE_SPAN, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "TRACEABLE_FAIL", (void *)_cffi_const_TRACEABLE_FAIL, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "TRACEABLE_FULL_SPAN", (void *)_cffi_const_TRACEABLE_FULL_SPAN, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "TRACEABLE_LOG_NONE", (void *)_cffi_const_TRACEABLE_LOG_NONE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "TRACEABLE_LOG_STDOUT", (void *)_cffi_const_TRACEABLE_LOG_STDOUT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "TRACEABLE_NO_SPAN", (void *)_cffi_const_TRACEABLE_NO_SPAN, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "TRACEABLE_SUCCESS", (void *)_cffi_const_TRACEABLE_SUCCESS, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "init_libtraceable_config", (void *)_cffi_f_init_libtraceable_config, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 47), (void *)_cffi_d_init_libtraceable_config },
  { "modsecurity_cleanup_rule_engine", (void *)_cffi_f_modsecurity_cleanup_rule_engine, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 29), (void *)_cffi_d_modsecurity_cleanup_rule_engine },
  { "modsecurity_cleanup_rule_matches", (void *)_cffi_f_modsecurity_cleanup_rule_matches, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 16), (void *)_cffi_d_modsecurity_cleanup_rule_matches },
  { "modsecurity_new_rule_engine", (void *)_cffi_f_modsecurity_new_rule_engine, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 12), (void *)_cffi_d_modsecurity_new_rule_engine },
  { "modsecurity_process_attributes", (void *)_cffi_f_modsecurity_process_attributes, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 32), (void *)_cffi_d_modsecurity_process_attributes },
  { "traceable_barespan_attributes", (void *)_cffi_f_traceable_barespan_attributes, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 37), (void *)_cffi_d_traceable_barespan_attributes },
  { "traceable_decode_protobuf", (void *)_cffi_f_traceable_decode_protobuf, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 6), (void *)_cffi_d_traceable_decode_protobuf },
  { "traceable_delete_barespan_attributes_result", (void *)_cffi_f_traceable_delete_barespan_attributes_result, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 19), (void *)_cffi_d_traceable_delete_barespan_attributes_result },
  { "traceable_delete_libtraceable", (void *)_cffi_f_traceable_delete_libtraceable, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 29), (void *)_cffi_d_traceable_delete_libtraceable },
  { "traceable_delete_process_request_result_data", (void *)_cffi_f_traceable_delete_process_request_result_data, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 26), (void *)_cffi_d_traceable_delete_process_request_result_data },
  { "traceable_is_content_type_capturable", (void *)_cffi_f_traceable_is_content_type_capturable, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_traceable_is_content_type_capturable },
  { "traceable_new_libtraceable", (void *)_cffi_f_traceable_new_libtraceable, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 22), (void *)_cffi_d_traceable_new_libtraceable },
  { "traceable_process_request", (void *)_cffi_f_traceable_process_request, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 42), (void *)_cffi_d_traceable_process_request },
  { "traceable_process_request_body", (void *)_cffi_f_traceable_process_request_body, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 42), (void *)_cffi_d_traceable_process_request_body },
  { "traceable_process_request_headers", (void *)_cffi_f_traceable_process_request_headers, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 42), (void *)_cffi_d_traceable_process_request_headers },
  { "traceable_start_libtraceable", (void *)_cffi_f_traceable_start_libtraceable, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 29), (void *)_cffi_d_traceable_start_libtraceable },
};

static const struct _cffi_field_s _cffi_fields[] = {
  { "rule_id", offsetof(modsecurity_rule_match, rule_id),
               sizeof(((modsecurity_rule_match *)0)->rule_id),
               _CFFI_OP(_CFFI_OP_NOOP, 52) },
  { "rule_message", offsetof(modsecurity_rule_match, rule_message),
                    sizeof(((modsecurity_rule_match *)0)->rule_message),
                    _CFFI_OP(_CFFI_OP_NOOP, 52) },
  { "match_message", offsetof(modsecurity_rule_match, match_message),
                     sizeof(((modsecurity_rule_match *)0)->match_message),
                     _CFFI_OP(_CFFI_OP_NOOP, 52) },
  { "match_attribute", offsetof(modsecurity_rule_match, match_attribute),
                       sizeof(((modsecurity_rule_match *)0)->match_attribute),
                       _CFFI_OP(_CFFI_OP_NOOP, 52) },
  { "paranoia_level", offsetof(modsecurity_rule_match, paranoia_level),
                      sizeof(((modsecurity_rule_match *)0)->paranoia_level),
                      _CFFI_OP(_CFFI_OP_NOOP, 3) },
  { "rule_uuid", offsetof(modsecurity_rule_match, rule_uuid),
                 sizeof(((modsecurity_rule_match *)0)->rule_uuid),
                 _CFFI_OP(_CFFI_OP_NOOP, 52) },
  { "count", offsetof(modsecurity_rule_matches, count),
             sizeof(((modsecurity_rule_matches *)0)->count),
             _CFFI_OP(_CFFI_OP_NOOP, 3) },
  { "match_arr", offsetof(modsecurity_rule_matches, match_arr),
                 sizeof(((modsecurity_rule_matches *)0)->match_arr),
                 _CFFI_OP(_CFFI_OP_NOOP, 55) },
  { "service_name", offsetof(traceable_agent_config, service_name),
                    sizeof(((traceable_agent_config *)0)->service_name),
                    _CFFI_OP(_CFFI_OP_NOOP, 1) },
  { "enabled", offsetof(traceable_api_discovery_config, enabled),
               sizeof(((traceable_api_discovery_config *)0)->enabled),
               _CFFI_OP(_CFFI_OP_NOOP, 3) },
  { "key", offsetof(traceable_attribute, key),
           sizeof(((traceable_attribute *)0)->key),
           _CFFI_OP(_CFFI_OP_NOOP, 1) },
  { "value", offsetof(traceable_attribute, value),
             sizeof(((traceable_attribute *)0)->value),
             _CFFI_OP(_CFFI_OP_NOOP, 1) },
  { "count", offsetof(traceable_attributes, count),
             sizeof(((traceable_attributes *)0)->count),
             _CFFI_OP(_CFFI_OP_NOOP, 3) },
  { "attribute_array", offsetof(traceable_attributes, attribute_array),
                       sizeof(((traceable_attributes *)0)->attribute_array),
                       _CFFI_OP(_CFFI_OP_NOOP, 61) },
  { "enabled", offsetof(traceable_blocking_config, enabled),
               sizeof(((traceable_blocking_config *)0)->enabled),
               _CFFI_OP(_CFFI_OP_NOOP, 3) },
  { "opa_config", offsetof(traceable_blocking_config, opa_config),
                  sizeof(((traceable_blocking_config *)0)->opa_config),
                  _CFFI_OP(_CFFI_OP_NOOP, 66) },
  { "modsecurity_config", offsetof(traceable_blocking_config, modsecurity_config),
                          sizeof(((traceable_blocking_config *)0)->modsecurity_config),
                          _CFFI_OP(_CFFI_OP_NOOP, 65) },
  { "rb_config", offsetof(traceable_blocking_config, rb_config),
                 sizeof(((traceable_blocking_config *)0)->rb_config),
                 _CFFI_OP(_CFFI_OP_NOOP, 67) },
  { "evaluate_body", offsetof(traceable_blocking_config, evaluate_body),
                     sizeof(((traceable_blocking_config *)0)->evaluate_body),
                     _CFFI_OP(_CFFI_OP_NOOP, 3) },
  { "skip_internal_request", offsetof(traceable_blocking_config, skip_internal_request),
                             sizeof(((traceable_blocking_config *)0)->skip_internal_request),
                             _CFFI_OP(_CFFI_OP_NOOP, 3) },
  { "max_recursion_depth", offsetof(traceable_blocking_config, max_recursion_depth),
                           sizeof(((traceable_blocking_config *)0)->max_recursion_depth),
                           _CFFI_OP(_CFFI_OP_NOOP, 3) },
  { "log_config", offsetof(traceable_libtraceable_config, log_config),
                  sizeof(((traceable_libtraceable_config *)0)->log_config),
                  _CFFI_OP(_CFFI_OP_NOOP, 64) },
  { "remote_config", offsetof(traceable_libtraceable_config, remote_config),
                     sizeof(((traceable_libtraceable_config *)0)->remote_config),
                     _CFFI_OP(_CFFI_OP_NOOP, 68) },
  { "blocking_config", offsetof(traceable_libtraceable_config, blocking_config),
                       sizeof(((traceable_libtraceable_config *)0)->blocking_config),
                       _CFFI_OP(_CFFI_OP_NOOP, 63) },
  { "agent_config", offsetof(traceable_libtraceable_config, agent_config),
                    sizeof(((traceable_libtraceable_config *)0)->agent_config),
                    _CFFI_OP(_CFFI_OP_NOOP, 59) },
  { "api_discovery_config", offsetof(traceable_libtraceable_config, api_discovery_config),
                            sizeof(((traceable_libtraceable_config *)0)->api_discovery_config),
                            _CFFI_OP(_CFFI_OP_NOOP, 60) },
  { "sampling_config", offsetof(traceable_libtraceable_config, sampling_config),
                       sizeof(((traceable_libtraceable_config *)0)->sampling_config),
                       _CFFI_OP(_CFFI_OP_NOOP, 69) },
  { "mode", offsetof(traceable_log_configuration, mode),
            sizeof(((traceable_log_configuration *)0)->mode),
            _CFFI_OP(_CFFI_OP_NOOP, 49) },
  { "enabled", offsetof(traceable_modsecurity_config, enabled),
               sizeof(((traceable_modsecurity_config *)0)->enabled),
               _CFFI_OP(_CFFI_OP_NOOP, 3) },
  { "enabled", offsetof(traceable_opa_config, enabled),
               sizeof(((traceable_opa_config *)0)->enabled),
               _CFFI_OP(_CFFI_OP_NOOP, 3) },
  { "opa_server_url", offsetof(traceable_opa_config, opa_server_url),
                      sizeof(((traceable_opa_config *)0)->opa_server_url),
                      _CFFI_OP(_CFFI_OP_NOOP, 1) },
  { "logging_dir", offsetof(traceable_opa_config, logging_dir),
                   sizeof(((traceable_opa_config *)0)->logging_dir),
                   _CFFI_OP(_CFFI_OP_NOOP, 1) },
  { "logging_file_prefix", offsetof(traceable_opa_config, logging_file_prefix),
                           sizeof(((traceable_opa_config *)0)->logging_file_prefix),
                           _CFFI_OP(_CFFI_OP_NOOP, 1) },
  { "cert_file", offsetof(traceable_opa_config, cert_file),
                 sizeof(((traceable_opa_config *)0)->cert_file),
                 _CFFI_OP(_CFFI_OP_NOOP, 1) },
  { "use_secure_connection", offsetof(traceable_opa_config, use_secure_connection),
                             sizeof(((traceable_opa_config *)0)->use_secure_connection),
                             _CFFI_OP(_CFFI_OP_NOOP, 3) },
  { "log_to_console", offsetof(traceable_opa_config, log_to_console),
                      sizeof(((traceable_opa_config *)0)->log_to_console),
                      _CFFI_OP(_CFFI_OP_NOOP, 3) },
  { "skip_verify", offsetof(traceable_opa_config, skip_verify),
                   sizeof(((traceable_opa_config *)0)->skip_verify),
                   _CFFI_OP(_CFFI_OP_NOOP, 3) },
  { "min_delay", offsetof(traceable_opa_config, min_delay),
                 sizeof(((traceable_opa_config *)0)->min_delay),
                 _CFFI_OP(_CFFI_OP_NOOP, 3) },
  { "max_delay", offsetof(traceable_opa_config, max_delay),
                 sizeof(((traceable_opa_config *)0)->max_delay),
                 _CFFI_OP(_CFFI_OP_NOOP, 3) },
  { "debug_log", offsetof(traceable_opa_config, debug_log),
                 sizeof(((traceable_opa_config *)0)->debug_log),
                 _CFFI_OP(_CFFI_OP_NOOP, 3) },
  { "block", offsetof(traceable_process_request_result, block),
             sizeof(((traceable_process_request_result *)0)->block),
             _CFFI_OP(_CFFI_OP_NOOP, 3) },
  { "span_type", offsetof(traceable_process_request_result, span_type),
                 sizeof(((traceable_process_request_result *)0)->span_type),
                 _CFFI_OP(_CFFI_OP_NOOP, 51) },
  { "propagate", offsetof(traceable_process_request_result, propagate),
                 sizeof(((traceable_process_request_result *)0)->propagate),
                 _CFFI_OP(_CFFI_OP_NOOP, 3) },
  { "attributes", offsetof(traceable_process_request_result, attributes),
                  sizeof(((traceable_process_request_result *)0)->attributes),
                  _CFFI_OP(_CFFI_OP_NOOP, 20) },
  { "enabled", offsetof(traceable_rangeblocking_config, enabled),
               sizeof(((traceable_rangeblocking_config *)0)->enabled),
               _CFFI_OP(_CFFI_OP_NOOP, 3) },
  { "enabled", offsetof(traceable_remote_config, enabled),
               sizeof(((traceable_remote_config *)0)->enabled),
               _CFFI_OP(_CFFI_OP_NOOP, 3) },
  { "remote_endpoint", offsetof(traceable_remote_config, remote_endpoint),
                       sizeof(((traceable_remote_config *)0)->remote_endpoint),
                       _CFFI_OP(_CFFI_OP_NOOP, 1) },
  { "poll_period_sec", offsetof(traceable_remote_config, poll_period_sec),
                       sizeof(((traceable_remote_config *)0)->poll_period_sec),
                       _CFFI_OP(_CFFI_OP_NOOP, 3) },
  { "cert_file", offsetof(traceable_remote_config, cert_file),
                 sizeof(((traceable_remote_config *)0)->cert_file),
                 _CFFI_OP(_CFFI_OP_NOOP, 1) },
  { "use_secure_connection", offsetof(traceable_remote_config, use_secure_connection),
                             sizeof(((traceable_remote_config *)0)->use_secure_connection),
                             _CFFI_OP(_CFFI_OP_NOOP, 3) },
  { "grpc_max_call_recv_msg_size", offsetof(traceable_remote_config, grpc_max_call_recv_msg_size),
                                   sizeof(((traceable_remote_config *)0)->grpc_max_call_recv_msg_size),
                                   _CFFI_OP(_CFFI_OP_NOOP, 54) },
  { "cert_string", offsetof(traceable_remote_config, cert_string),
                   sizeof(((traceable_remote_config *)0)->cert_string),
                   _CFFI_OP(_CFFI_OP_NOOP, 1) },
  { "enabled", offsetof(traceable_sampling_config, enabled),
               sizeof(((traceable_sampling_config *)0)->enabled),
               _CFFI_OP(_CFFI_OP_NOOP, 3) },
};

static const struct _cffi_struct_union_s _cffi_struct_unions[] = {
  { "$modsecurity_rule_match", 56, _CFFI_F_CHECK_FIELDS,
    sizeof(modsecurity_rule_match), offsetof(struct _cffi_align_typedef_modsecurity_rule_match, y), 0, 6 },
  { "$modsecurity_rule_matches", 17, _CFFI_F_CHECK_FIELDS,
    sizeof(modsecurity_rule_matches), offsetof(struct _cffi_align_typedef_modsecurity_rule_matches, y), 6, 2 },
  { "$traceable_agent_config", 59, _CFFI_F_CHECK_FIELDS,
    sizeof(traceable_agent_config), offsetof(struct _cffi_align_typedef_traceable_agent_config, y), 8, 1 },
  { "$traceable_api_discovery_config", 60, _CFFI_F_CHECK_FIELDS,
    sizeof(traceable_api_discovery_config), offsetof(struct _cffi_align_typedef_traceable_api_discovery_config, y), 9, 1 },
  { "$traceable_attribute", 62, _CFFI_F_CHECK_FIELDS,
    sizeof(traceable_attribute), offsetof(struct _cffi_align_typedef_traceable_attribute, y), 10, 2 },
  { "$traceable_attributes", 20, _CFFI_F_CHECK_FIELDS,
    sizeof(traceable_attributes), offsetof(struct _cffi_align_typedef_traceable_attributes, y), 12, 2 },
  { "$traceable_blocking_config", 63, _CFFI_F_CHECK_FIELDS,
    sizeof(traceable_blocking_config), offsetof(struct _cffi_align_typedef_traceable_blocking_config, y), 14, 7 },
  { "$traceable_libtraceable_config", 23, _CFFI_F_CHECK_FIELDS,
    sizeof(traceable_libtraceable_config), offsetof(struct _cffi_align_typedef_traceable_libtraceable_config, y), 21, 6 },
  { "$traceable_log_configuration", 64, _CFFI_F_CHECK_FIELDS,
    sizeof(traceable_log_configuration), offsetof(struct _cffi_align_typedef_traceable_log_configuration, y), 27, 1 },
  { "$traceable_modsecurity_config", 65, _CFFI_F_CHECK_FIELDS,
    sizeof(traceable_modsecurity_config), offsetof(struct _cffi_align_typedef_traceable_modsecurity_config, y), 28, 1 },
  { "$traceable_opa_config", 66, _CFFI_F_CHECK_FIELDS,
    sizeof(traceable_opa_config), offsetof(struct _cffi_align_typedef_traceable_opa_config, y), 29, 11 },
  { "$traceable_process_request_result", 27, _CFFI_F_CHECK_FIELDS,
    sizeof(traceable_process_request_result), offsetof(struct _cffi_align_typedef_traceable_process_request_result, y), 40, 4 },
  { "$traceable_rangeblocking_config", 67, _CFFI_F_CHECK_FIELDS,
    sizeof(traceable_rangeblocking_config), offsetof(struct _cffi_align_typedef_traceable_rangeblocking_config, y), 44, 1 },
  { "$traceable_remote_config", 68, _CFFI_F_CHECK_FIELDS,
    sizeof(traceable_remote_config), offsetof(struct _cffi_align_typedef_traceable_remote_config, y), 45, 7 },
  { "$traceable_sampling_config", 69, _CFFI_F_CHECK_FIELDS,
    sizeof(traceable_sampling_config), offsetof(struct _cffi_align_typedef_traceable_sampling_config, y), 52, 1 },
};

static const struct _cffi_enum_s _cffi_enums[] = {
  { "$TRACEABLE_LOG_MODE", 49, _cffi_prim_int(sizeof(TRACEABLE_LOG_MODE), ((TRACEABLE_LOG_MODE)-1) <= 0),
    "TRACEABLE_LOG_NONE,TRACEABLE_LOG_STDOUT" },
  { "$TRACEABLE_RET", 50, _cffi_prim_int(sizeof(TRACEABLE_RET), ((TRACEABLE_RET)-1) <= 0),
    "TRACEABLE_SUCCESS,TRACEABLE_FAIL" },
  { "$TRACEABLE_SPAN_TYPE", 51, _cffi_prim_int(sizeof(TRACEABLE_SPAN_TYPE), ((TRACEABLE_SPAN_TYPE)-1) <= 0),
    "TRACEABLE_NO_SPAN,TRACEABLE_BARE_SPAN,TRACEABLE_FULL_SPAN" },
};

static const struct _cffi_typename_s _cffi_typenames[] = {
  { "TRACEABLE_LOG_MODE", 49 },
  { "TRACEABLE_RET", 50 },
  { "TRACEABLE_SPAN_TYPE", 51 },
  { "int16_t", 57 },
  { "int32_t", 3 },
  { "int64_t", 54 },
  { "int8_t", 58 },
  { "int_fast16_t", 3 },
  { "int_fast32_t", 3 },
  { "int_fast64_t", 54 },
  { "int_fast8_t", 58 },
  { "int_least16_t", 57 },
  { "int_least32_t", 3 },
  { "int_least64_t", 54 },
  { "int_least8_t", 58 },
  { "intmax_t", 54 },
  { "intptr_t", 54 },
  { "modsecurity_rule_engine", 30 },
  { "modsecurity_rule_match", 56 },
  { "modsecurity_rule_matches", 17 },
  { "traceable_agent_config", 59 },
  { "traceable_api_discovery_config", 60 },
  { "traceable_attribute", 62 },
  { "traceable_attributes", 20 },
  { "traceable_blocking_config", 63 },
  { "traceable_libtraceable", 30 },
  { "traceable_libtraceable_config", 23 },
  { "traceable_log_configuration", 64 },
  { "traceable_modsecurity_config", 65 },
  { "traceable_opa_config", 66 },
  { "traceable_process_request_result", 27 },
  { "traceable_rangeblocking_config", 67 },
  { "traceable_remote_config", 68 },
  { "traceable_sampling_config", 69 },
  { "uint16_t", 73 },
  { "uint32_t", 71 },
  { "uint64_t", 72 },
  { "uint8_t", 70 },
  { "uint_fast16_t", 71 },
  { "uint_fast32_t", 71 },
  { "uint_fast64_t", 72 },
  { "uint_fast8_t", 70 },
  { "uint_least16_t", 73 },
  { "uint_least32_t", 71 },
  { "uint_least64_t", 72 },
  { "uint_least8_t", 70 },
  { "uintmax_t", 72 },
  { "uintptr_t", 72 },
};

static const struct _cffi_type_context_s _cffi_type_context = {
  _cffi_types,
  _cffi_globals,
  _cffi_fields,
  _cffi_struct_unions,
  _cffi_enums,
  _cffi_typenames,
  23,  /* num_globals */
  15,  /* num_struct_unions */
  3,  /* num_enums */
  48,  /* num_typenames */
  NULL,  /* no includes */
  75,  /* num_types */
  0,  /* flags */
};

#ifdef __GNUC__
#  pragma GCC visibility push(default)  /* for -fvisibility= */
#endif

#ifdef PYPY_VERSION
PyMODINIT_FUNC
_cffi_pypyinit__libtraceable(const void *p[])
{
    p[0] = (const void *)0x2601;
    p[1] = &_cffi_type_context;
#if PY_MAJOR_VERSION >= 3
    return NULL;
#endif
}
#  ifdef _MSC_VER
     PyMODINIT_FUNC
#  if PY_MAJOR_VERSION >= 3
     PyInit__libtraceable(void) { return NULL; }
#  else
     init_libtraceable(void) { }
#  endif
#  endif
#elif PY_MAJOR_VERSION >= 3
PyMODINIT_FUNC
PyInit__libtraceable(void)
{
  return _cffi_init("traceableai.filter._libtraceable", 0x2601, &_cffi_type_context);
}
#else
PyMODINIT_FUNC
init_libtraceable(void)
{
  _cffi_init("traceableai.filter._libtraceable", 0x2601, &_cffi_type_context);
}
#endif

#ifdef __GNUC__
#  pragma GCC visibility pop
#endif
